/**
 * Copyright (C) 1998 Stewart Allen (stewart@neuron.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import java.util.*;

public class Connection
{
	int NextSeqNumber;
	int SubsysId;
	int Flags;

	int SecurityLevel;
	int SessionKey;
	int EncryptionType;

	int LocalHandle;
	int RemoteHandle;

	// ---

	int state;
	byte clientIdent[];
	long startTime;
	Endpoint remoteHost = null;
	Hashtable mutexs = new Hashtable();

	// --- 

	Connection(int LocalHandle)
	{
		this.LocalHandle = LocalHandle;
		this.NextSeqNumber = 0;
		this.state = 0;
		this.startTime = System.currentTimeMillis();
	}

	public int getTimeStamp()
	{
		return (int)(System.currentTimeMillis() - startTime);
	}

	public String toString()
	{
		return "["+
			LocalHandle+"-->"+RemoteHandle+
			",state="+state+
			",user="+(clientIdent != null ? new String(clientIdent) : "")+
			"]";
	}

	// --- 
	
	public void addMutex()
	{
		addMutex("rpc2");
	}

	public void addMutex(String name)
	{
		mutexs.put(name, new Mutex());
	}

	public void delMutex(String name)
	{
		mutexs.remove(name);
	}

	public Object waitMutex()
	{
		return waitMutex("rpc2");
	}

	public Object waitMutex(String name)
	{
		Mutex m = (Mutex)mutexs.get(name);
		if (m != null)
			return m.waitRelease();
		return null;
	}

	public void notifyMutex(RPC2_Packet packet)
	{
		notifyMutex("rpc2", packet);
	}

	public void notifyMutex(String name, Object o)
	{
		Mutex m = (Mutex)mutexs.get(name);
		if (m != null)
		{
			m.release(o);
			mutexs.remove(name);
		}
	}

}

