/**
 * Copyright (C) 1998 Stewart Allen (stewart@neuron.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import java.io.*;
import java.net.*;

public class Listener extends Thread
{
	int port;
	DatagramSocket s;

	public Listener(int port)
	{
		this.port = port;
		try {
			if (port <= 0) {
				s = new DatagramSocket();
				this.port = s.getLocalPort();
			} else {
				s = new DatagramSocket(port);
			}
			Util.log("new listener on "+port);
		} catch (Exception e) { e.printStackTrace(); }
	}

	public static Listener newListener()
	{
		return newListener(0);
	}

	public static Listener newListener(int port)
	{
		Listener l = new Listener(port);
		l.start();
		return l;
	}

	public void sendPacketReliably(Connection c, RPC2_Packet packet)
	{
		try {
			c.NextSeqNumber++;
			sendPacket(c.remoteHost.host, c.remoteHost.port, packet);
		} catch (Exception e) { e.printStackTrace(); }
	}

	public void sendPacket(InetAddress host, int port, RPC2_Packet packet)
		throws IOException
	{
		Util.log(">>> --- ("+port+") --------------------------------");
		Util.log(packet.toString());

		byte data[] = packet.getPacketData();
		DatagramPacket p =
			new DatagramPacket(data, data.length, host, port);
		s.send(p);
	}

	public void run()
	{
		try {
			while (true)
			{
				byte data[] = new byte[4500];
				DatagramPacket dp = new DatagramPacket(data, 4500);
				s.receive(dp);

				int len = dp.getLength();
				byte ndata[] = new byte[len];
				System.arraycopy(dp.getData(),0,ndata,0,len);

				RPC2_Packet packet = new RPC2_Packet(ndata);
				RPC2_Server subSys = SystemManager.getSystem(packet.SubsysId);
				if (subSys != null)
					subSys.handle(packet, this);
				else
					Util.log("Unknown subsystem: "+packet.SubsysId);
			}
		} catch (Exception e) { e.printStackTrace(); }
	}
}

