/**
 * Copyright (C) 1998 Stewart Allen (stewart@neuron.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import java.io.*;
import java.net.*;
import java.util.*;

public class RPC2
{
	public static byte[] Version =
		RPC2_Util.padString("Version 14.0: Satya, 6 May 1988, 10:00",0,96).getBytes();
//		RPC2_Util.padString("1.0 Beta, (C) Copyright 1998 Stewart Allen",0,96).getBytes();
	public static InetAddress LocalHost;

	static {
		try {
			LocalHost = InetAddress.getLocalHost();
		} catch (Exception e) { e.printStackTrace(); }
	}

	public static final int RPC2_PROTOVERSION	 = 8;

	public static final int RPC2_OPENKIMONO		 = 98; // no crypto, no auth
	public static final int RPC2_AUTHONLY		 = 12; // no crypto, auth
	public static final int RPC2_HEADERSONLY	 = 73; // crypt hdr, auth
	public static final int RPC2_SECURE			 = 66; // crypt, auth

	public static final int RPC2_DES			 = 1;
	public static final int RPC2_XOR			 = 2;
	public static final int RPC2_ENCRYPTIONTYPES = RPC2_DES | RPC2_XOR;

	public static final int RPC2_KEYSIZE		 = 8;  // crypto key size
	public static final int RPC2_SUCCESS		 = 0;
	public static final int RPC2_WLIMIT			 = -1;
	public static final int RPC2_ELIMIT			 = -1000;
	public static final int RPC2_FLIMIT			 = -2000;

	public static final int RPC2_OLDVERSION		 = RPC2_WLIMIT-1;
	public static final int RPC2_INVALIDOPCODE	 = RPC2_WLIMIT-2;
	public static final int RPC2_BADDATA		 = RPC2_WLIMIT-3;
	public static final int RPC2_NOGREEDY		 = RPC2_WLIMIT-4;
	public static final int	RPC2_ABANDONED		 = RPC2_WLIMIT-5;
	public static final int RPC2_CONNBUSY		 = RPC2_ELIMIT-1;
	public static final int RPC2_SEFAIL1		 = RPC2_ELIMIT-2;
	public static final int RPC2_TOOLONG		 = RPC2_ELIMIT-3;
	public static final int	RPC2_NOMGROUP		 = RPC2_ELIMIT-4;
	public static final int RPC2_MGRPBUSY		 = RPC2_ELIMIT-5;
	public static final int	RPC2_NOTGROUPMEMBER	 = RPC2_ELIMIT-6;
	public static final int	RPC2_DUPLICATEMEMBER = RPC2_ELIMIT-7;
	public static final int	RPC2_BADMGROUP		 = RPC2_ELIMIT-8;
	public static final int RPC2_FAIL			 = RPC2_FLIMIT-1;
	public static final int RPC2_NOCONNECTION	 = RPC2_FLIMIT-2;
	public static final int RPC2_TIMEOUT		 = RPC2_FLIMIT-3;
	public static final int RPC2_NOBINDING		 = RPC2_FLIMIT-4;
	public static final int RPC2_DUPLICATESERVER = RPC2_FLIMIT-5;
	public static final int RPC2_NOTWORKER		 = RPC2_FLIMIT-6;
	public static final int RPC2_NOTCLIENT		 = RPC2_FLIMIT-7;
	public static final int RPC2_WRONGVERSION	 = RPC2_FLIMIT-8;
	public static final int RPC2_NOTAUTHENTICATED= RPC2_FLIMIT-9;
	public static final int RPC2_CLOSECONNECTION = RPC2_FLIMIT-10;
	public static final int RPC2_BADFILTER		 = RPC2_FLIMIT-11;
	public static final int RPC2_LWPNOTINIT		 = RPC2_FLIMIT-12;
	public static final int RPC2_BADSERVER		 = RPC2_FLIMIT-13;
	public static final int RPC2_SEFAIL2		 = RPC2_FLIMIT-14;
	public static final int RPC2_DEAD			 = RPC2_FLIMIT-15;
	public static final int RPC2_NAKED			 = RPC2_FLIMIT-16;
	public static final int RPC2_SEFAIL3		 = RPC2_FLIMIT-17; // for side effects
	public static final int RPC2_SEFAIL4		 = RPC2_FLIMIT-18; // for side effects

	public static String getOpcodeName(int Opcode)
	{
		switch (Opcode)
		{
			// base Opcodes
			case RPC2_INIT1OPENKIMONO:	return "RPC2_INIT1OPENKIMONO";
			case RPC2_INIT1AUTHONLY:	return "RPC2_INIT1AUTHONLY";
			case RPC2_INIT1HEADERSONLY:	return "RPC2_INIT1HEADERSONLY";
			case RPC2_INIT1SECURE:		return "RPC2_INIT1SECURE";
			case RPC2_LASTACK:			return "RPC2_LASTACK";
			case RPC2_REPLY:			return "RPC2_REPLY";
			case RPC2_INIT2:			return "RPC2_INIT2";
			case RPC2_INIT3:			return "RPC2_INIT3";
			case RPC2_INIT4:			return "RPC2_INIT4";
			case RPC2_NEWCONNECTION:	return "RPC2_NEWCONNECTION";
			case RPC2_BUSY:				return "RPC2_BUSY";
			case RPC2_INITMULTICAST:	return "RPC2_INITMULTICAST";

			// test Opcodes
			case TEST_PING:				return "TEST_PING";
			case TEST_PINGMANY:			return "TEST_PINGMANY";
			case TEST_FETCH:			return "TEST_FETCH";
			case TEST_STORE:			return "TEST_STORE";
			case TEST_REMOTESTATS:		return "TEST_REMOTESTATS";
			default: return null;
		}
	}

	// Opcodes for internal subsystem (1001)
	public static final int RPC2_INIT1OPENKIMONO = -2;   // new conn with RPC2_OPENKIMONO
	public static final int RPC2_INIT1AUTHONLY	 = -3;   // new conn with RPC2_AUTHONLY
	public static final int RPC2_INIT1HEADERSONLY= -4;   // new conn with RPC2_HEADERSONLY
	public static final int RPC2_INIT1SECURE	 = -5;   // new conn with RPC2_SECURE
	public static final int RPC2_LASTACK		 = -6;   // ACK packet
	public static final int RPC2_REPLY			 = -8;   // REPLY packet
	public static final int RPC2_INIT2			 = -10;  // Phase 2 of bind handshake 
	public static final int RPC2_INIT3			 = -11;  // Phase 3 of bind handshake 
	public static final int RPC2_INIT4			 = -12;  // Phase 4 of bind handshake 
	public static final int RPC2_NEWCONNECTION	 = -13;  // opcode of fake request generated by RPC2_GetRequest() on new connection */
	public static final int RPC2_BUSY			 = -14;  // KEEPALIVE packet
	public static final int	RPC2_INITMULTICAST	 = -15;  // new Multicast connection
	public static final int RPC2_MAXPACKETSIZE	 = 4500; // size of largest packet
	public static final int RPC2_RETRY			 = 0x1;  // set by runtime
	public static final int RPC2_ENCRYPTED		 = 0x2;	 // set by runtime
	public static final int	RPC2_MULTICAST		 = 0x4;	 // set by runtime
	public static final int RPC2_MAXLOGLENGTH	 = 32;

	// Opcodes for test client/server
	public final static int TEST_PING = 4;
	public final static int TEST_PINGMANY = 5;
	public final static int TEST_FETCH = 6;
	public final static int TEST_STORE = 7;
	public final static int TEST_REMOTESTATS = 10;

	public static void main(String args[])
		throws Exception
	{
		if (args.length < 1)
			usage();

		if (args[0].equals("-s")) {
			SystemManager.registerSystem(1001, new RPC2_TestServer());
			SystemManager.registerSystem(1189, new SFTP_Server());
			new Listener(1700).start();
		} else if (args[0].equals("-c")) {
			RPC2_Client c = new RPC2_Client();
			String cmd;
			do {
				cmd = getLine("command (?=help) ");
				if (cmd.equals("?")) {
					System.out.println(
						"c = connect     ? = help     p = ping    r = start stats\n" +
						"q = quit"
					);
				} else
				if (cmd.equals("c")) {
					String host = getLine(" host? ");
					String port = getLine(" port? ");
					String Sef = getLine(" side effect (0 or 1189)? ");
					int sef = (Sef.length() == 0 ? 0 : Integer.parseInt(Sef));
					String Sec = getLine(" security (98, 12, 73, 66)? ");
					int sec = (Sec.length() == 0 ? 0 : Integer.parseInt(Sec));
					String id  = getLine(" identity? ");
					c.connect(host, Integer.parseInt(port), 1001, sef, sec, id);
				} else
				if (cmd.equals("r")) {
					c.remoteStats();
				} else
				if (cmd.equals("p")) {
					c.ping();
				} else
				if (cmd.equals("f")) {
					String rname = getLine(" remote file name? ");
					String lname = getLine(" local file name? ");
					String file = c.fetch(rname);
				}// else
			} while (cmd != null && !cmd.equals("q"));
			System.exit(0);
		} else {
			usage();
		}
	}

	static String getLine()
	{
		return getLine("");
	}

	static String getLine(String prompt)
	{
		System.out.print(prompt);
		try {
			return new BufferedReader(new InputStreamReader(System.in)).readLine();
		} catch (Exception e) { return null; }
	}

	public static void usage()
	{
		System.err.println("-s = test server");
		System.err.println("-c = test client");
		System.exit(1);
	}
}

