/**
 * Copyright (C) 1998 Stewart Allen (stewart@neuron.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import java.io.*;
import java.util.*;

public abstract class RPC2_BodyStruct
{
	static int indent = 0;
	static String pad = "";

	RPC2_Packet packet;
	String name;
	int headPos;
	int longestName;
	Vector printFields;

	// for CONSTRUCTING a packet
	public RPC2_BodyStruct()
	{
	}

	public RPC2_BodyStruct(String name)
	{
		this.name = name;
	}

	// for READING a packet
	public RPC2_BodyStruct(RPC2_Packet packet, String name)
	{
		this.packet = packet;
		this.headPos = packet.getPos();
		this.name = name;
	}

	// for WRITING a packet
	public abstract void encode(RPC2_Packet packet)
		;

	class PrintField
	{
		String name;
		String value;

		PrintField(String name, String value)
		{
			this.name = name;
			this.value = value;
		}
	}

	String startTag()
	{
//		return pad + "<> "+name+" @ "+headPos+" <>";
//		return "<RPC2 Body> "+name;
		return "<"+name+" @ "+headPos+">";
	}

	String endTag()
	{
		return "";
//		return pad + "<> "+name+" @ "+packet.getPos()+" <>";
	}

	static String pad()
	{
		return pad;
	}

	void startPrint()
	{
		printFields = new Vector();
		indent(1);
	}

	void addField(String name, int value)
	{
		addField(name, new Integer(value));
	}

	void addField(String name, long value)
	{
		addField(name, new Long(value));
	}

	void addField(String name, Object value)
	{
		printFields.addElement(new PrintField(name, value.toString()));
		longestName = Math.max(name.length(), longestName);
	}

	String getPrint()
	{
		StringBuffer sb = new StringBuffer(startTag());
		for (int i=0; i<printFields.size(); i++)
		{
			PrintField pf = (PrintField)printFields.elementAt(i);
			sb.append(pad + "  " + RPC2_Util.padString(pf.name, longestName) + " = " + pf.value);
		}
		sb.append(endTag());
		indent(-1);
		return sb.toString();
	}

	static void indent(int incr)
	{
		indent += incr;
		StringBuffer sb = new StringBuffer("\n");
		for (int i=0; i<indent-1; i++)
			sb.append("  ");
		pad = sb.toString();
	}
}

