/**
 * Copyright (C) 1998 Stewart Allen (stewart@neuron.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import java.net.*;
import java.util.*;

public class RPC2_Client
{
	Listener listener;
	Connection conn;

	public RPC2_Client()
	{
		listener = Listener.newListener();
		conn = ConnectionManager.newConnection();
		SystemManager.registerSystem(1189, new SFTP_Server());
	}

	public RPC2_Packet MakeRPC(RPC2_Packet in)
	{
		try {
			conn.addMutex();
			listener.sendPacketReliably(conn, in);
			return (RPC2_Packet)conn.waitMutex();
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	public String fetch(String name)
	{
		RPC2_Packet rpc = new RPC2_Packet(conn);
		rpc.Opcode = RPC2.TEST_FETCH;

		RPC2_XferInit b = new RPC2_XferInit(0, 0, -1, name, listener.port);
		rpc.setBody(b);

		// debugging
		//Util.log(b.toString());
		rpc.print();

		RPC2_Packet packet = MakeRPC(rpc);
		do {
			packet.print();
		} while ( (packet.SEFlags & SFTP.ALLOVER) == 0 && (packet.ReturnCode == SE_Descriptor.SE_SUCCESS) );

		return null;
	}

	public void ping()
	{
		// construct outbound packet
		RPC2_Packet rpc = new RPC2_Packet(conn);
		rpc.Opcode = RPC2.TEST_PING;
		// send rpc
		MakeRPC(rpc);
	}

	public void pingMany(int count)
	{
		// loop count times
		for (int i=0; i<count; i++)
		{
			// construct outbound packet
			RPC2_Packet rpc = new RPC2_Packet(conn);
			rpc.Opcode = RPC2.TEST_PINGMANY;
			// send rpc
			MakeRPC(rpc);
		}
	}

	public void remoteStats()
	{
		// construct outbound packet
		RPC2_Packet rpc = new RPC2_Packet(conn);
		rpc.Opcode = RPC2.TEST_REMOTESTATS;

		MakeRPC(rpc);
	}

	public void connect(String host, int port, int subSys,
						int sideEffect, int security, String identity)
	{
		// register "callback" mechanism
		SystemManager.registerSystem(1001, new RPC2_Server());

		// set subsystem on remote system
		conn.SubsysId = subSys;

		// construct outbound packet
		RPC2_Packet rpc = new RPC2_Packet(conn);

		rpc.LocalHandle = conn.LocalHandle;
		rpc.Flags = 14286848;
		rpc.Opcode = RPC2.RPC2_INIT1OPENKIMONO;
		rpc.TimeStamp = 3;
		rpc.BindTime = 0;

		RPC2_Init1Body b = new RPC2_Init1Body();
		b.Version = RPC2.Version;
		b.SenderHost = new RPC2_HostIdent(RPC2.LocalHost);
		b.SenderPortal = new RPC2_PortalIdent(listener.port);
		b.FakeBody =
			new RPC2_NewConnectionBody(sideEffect, RPC2.RPC2_OPENKIMONO, security, identity.getBytes());
			// side-effect, secure-mode, encryption-type, client-id

		rpc.setBody(b);

		// set remoteHost
		try {
			conn.remoteHost = new Endpoint(InetAddress.getByName(host), port);
		} catch (Exception e) { e.printStackTrace(); }

		MakeRPC(rpc);

		Util.log("Client connect complete");
	}
}

