/**
 * Copyright (C) 1998 Stewart Allen (stewart@neuron.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import java.net.*;

public class RPC2_HostIdent extends RPC2_BodyStruct
{
	public final static int RPC2_HOSTBYINETADDR = 17;
	public final static int RPC2_HOSTBYNAME = 39;
	public final static int RPC2_DUMMYHOST = 88888;

	// size of C Union Value
	final static int ValueSize = 64;

	int Tag;
	byte Value[];

	// for CONSTRUCTING a packet
	public RPC2_HostIdent(InetAddress addr)
	{
		super("RPC2_BodyStruct");
		Tag = RPC2_HOSTBYINETADDR;
		Value = RPC2_Util.padBytes(addr.getAddress(),0,ValueSize);
	}

	// for READING a packet
	public RPC2_HostIdent(RPC2_Packet packet)
	{
		super(packet, "RPC2_BodyStruct");
		Tag = packet.getInt();
		Value = packet.getBytes(ValueSize);
	}

	// for WRITING a packet
	public void encode(RPC2_Packet packet)
	{
		packet.put(Tag);
		packet.put(Value, false);
	}

	public InetAddress getAddress()
	{
		switch (Tag)
		{
			case RPC2_HOSTBYINETADDR:
				String name =
					Integer.toString((int)Value[0]) + "." +
					Integer.toString((int)Value[1]) + "." +
					Integer.toString((int)Value[2]) + "." +
					Integer.toString((int)Value[3]);
				try {
					return InetAddress.getByName(name);
				} catch (Exception e) {
					e.printStackTrace();
					return null;
				}
			default:
				return null;
		}
	}

	public String toString()
	{
		startPrint();
		addField("Tag", Tag);
//		addField("Value", Value.length);
		addField("Value", RPC2_Util.getHex(Value));
		return getPrint();
	}
}

