/**
 * Copyright (C) 1998 Stewart Allen (stewart@neuron.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

public class RPC2_PortalIdent extends RPC2_BodyStruct
{
	public final static int RPC2_PORTALBYINETNUMBER = 53;
	public final static int RPC2_PORTALBYNAME = 64;
	public final static int RPC2_DUMMYPORTAL = 99999;

	// size of C Union Struct
	final static int ValueSize = 20;

	int Tag;
	byte Value[];

	// for CONSTRUCTING a packet
	public RPC2_PortalIdent(int portNum)
	{
		super("RPC2_PortalIdent");
		Tag = RPC2_PORTALBYINETNUMBER;
		Value = RPC2_Util.padBytes(RPC2_Util.getInt2(portNum),0,ValueSize);
	}

	// for READING a packet
	public RPC2_PortalIdent(RPC2_Packet packet)
	{
		super(packet, "RPC2_PortalIdent");
		Tag = packet.getInt();
		Value = packet.getBytes(ValueSize);
	}

	// for WRITING a packet
	public void encode(RPC2_Packet packet)
	{
		packet.put(Tag);
		packet.put(Value, false);
	}

	public int getValue()
	{
		switch (Tag)
		{
			case RPC2_PORTALBYINETNUMBER:
				int val = 0;
				for (int i=0; i<2; i++)
					val |= ( (Value[i] & 0xff) << ((1-i)*8) );
				return val;
			default:
				return 0;
		}
	}

	public String toString()
	{
		startPrint();
		addField("Tag", Tag);
		addField("Value", getValue());
		addField("HexValue", RPC2_Util.getHex(Value));
		return getPrint();
	}
}

