/**
 * Copyright (C) 1998 Stewart Allen (stewart@neuron.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import java.net.*;
import java.util.*;

public class RPC2_Server
{
	public boolean processRequest(Connection conn, RPC2_Packet packet, Listener listener)
	{
		return false;
	}

	public boolean handle(RPC2_Packet packet, Listener listener)
	{
		Util.log("<<< --- ("+listener.port+") --------------------------------");
		Util.log(packet.toString());

		// pointer to Connection for incoming packet
		Connection c;

		// pointer for outbound replies
		RPC2_Packet out;

		// anything but OPEN will yield non-null
		c = ConnectionManager.getConnection(packet.RemoteHandle);
		if (c != null)
			c.NextSeqNumber = packet.SeqNumber+1;

		// if the Opcode is < 0 then it's a reserved system call
		if (packet.Opcode < 0) {
			switch (packet.Opcode)
			{
				// request for new connection
				case RPC2.RPC2_INIT1OPENKIMONO:
					// decode body as Init1Body and get FakeBody pointer
					RPC2_Init1Body init1 = new RPC2_Init1Body(packet);
					RPC2_NewConnectionBody fb = init1.FakeBody;

					// create a new "unconnected" connection
					c = ConnectionManager.newConnection();

					// set connection parameters
					c.SecurityLevel = fb.SecurityLevel;
					c.EncryptionType = fb.EncryptionType;
					c.RemoteHandle = packet.LocalHandle;
					// more internal connection stuff
					c.remoteHost = new Endpoint(
						init1.SenderHost.getAddress(),
						init1.SenderPortal.getValue() );
					c.clientIdent = fb.ClientIdent;

					// create reply packet and send
					out = new RPC2_Packet(packet, c);
					out.Opcode = RPC2.RPC2_INIT2;
					out.LocalHandle = c.LocalHandle;
					out.Uniquefier = packet.Uniquefier;
					out.ReturnCode = RPC2.RPC2_SUCCESS;
					out.setBody(new RPC2_Init2Body(0,0));
					listener.sendPacketReliably(c, out);

					Util.log("new connection: "+c);

					return true;

				case RPC2.RPC2_REPLY:
					c.notifyMutex(packet);

					return true;

				case RPC2.RPC2_INIT2:
					//decode body as Init2Body
					RPC2_Init2Body init2 = new RPC2_Init2Body(packet);

					// complete connection setup
					c.RemoteHandle = packet.LocalHandle;
					c.NextSeqNumber = 0;

					// debugging ...
					//Util.log(init2.toString());

					// notify client of completed setup
					c.notifyMutex(packet);

					return true;

				default:
					// INVALID system request
					packet.print();

					return false;
			}
		} else {
			// give the subclass a shot at handling the request
			boolean handled = processRequest(c, packet, listener);

			// if it's not handled, print it out for debugging
			if (!handled)
				packet.print();

			return handled;
		}

	}
}

