/**
 * Copyright (C) 1998 Stewart Allen (stewart@neuron.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import java.net.*;
import java.util.*;

public class RPC2_TestServer extends RPC2_Server
{
	public final static int SubsysId = 1001;

	public boolean processRequest(Connection conn, RPC2_Packet packet, Listener listener)
	{
		// pointer for outbound replies
		RPC2_Packet out;

		switch (packet.Opcode)
		{
			case RPC2.TEST_PING: // current enum value for PING
			case RPC2.TEST_PINGMANY: // current enum value for PINGMANY
			case RPC2.TEST_REMOTESTATS: // current enum value for REMOTESTATS
				// construct reply
				out = new RPC2_Packet(packet, conn);
				out.Opcode = RPC2.RPC2_REPLY;
				out.ReturnCode = RPC2.RPC2_SUCCESS;

				listener.sendPacketReliably(conn, out);
				
				return true;

			case RPC2.TEST_FETCH: // current enum value for FETCH (file-xfer)
				//decode body as XferInit
				RPC2_XferInit fetch = new RPC2_XferInit(packet);

				// debug ...
				Util.log(fetch.toString());
				packet.print();

				// construct reply
				out = new RPC2_Packet(packet, conn);
				out.Opcode = RPC2.RPC2_REPLY;
				out.ReturnCode = SE_Descriptor.SE_SUCCESS;

				out.SEFlags = SFTP.MOREDATA;
				//out.SEFlags = SFTP.PIGGY | SFTP.ALLOVER;
/*
				class FileData extends RPC2_BodyStruct
				{
					byte data[];
					FileData(byte data[]) { this.data = data; }
					public void encode(RPC2_Packet packet)
					{
						packet.put(data, false);
					}
				}

				out.setBody(new FileData("yo man!".getBytes()));
*/

				listener.sendPacketReliably(conn, out);
				
				return true;
			default:

				return false;
		}
	}
}

