/**
 * Copyright (C) 1998 Stewart Allen (stewart@neuron.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import java.util.*;

public class RPC2_XferInit extends RPC2_BodyStruct
{
	int Length;
	int Offset;
	int Quota;
	String Filename;
	SFTP_Parms parms;

	// for CONSTRUCTING a packet
	public RPC2_XferInit(int Length, int Offset, int Quota, String Filename, int portal)
	{
		super("RPC2_XferInit");
		this.Length = Length;
		this.Offset = Offset;
		this.Quota = Quota;
		this.Filename = Filename;
		this.parms = new SFTP_Parms(portal);
	}

	// for READING a packet
	public RPC2_XferInit(RPC2_Packet packet)
	{
		super(packet, "RPC2_XferInit");
		Length = packet.getInt();
		Offset = packet.getInt();
		Quota = packet.getInt();
		packet.getOneByte(); // skip "\n"
		Filename = packet.getNullTermString();
		parms = new SFTP_Parms(packet);
	}

	// for WRITING a packet
	public void encode(RPC2_Packet packet)
	{
		packet.put(Length);
		packet.put(Offset);
		packet.put(Quota);
		if (Filename != null)
			packet.put("\n"+Filename+"\0", false);
		else
			packet.put("\0", false);

		// piggyback SFTP_Parms onto xfer init packet
		packet.markSEDataOffset();
		packet.SEFlags = SFTP.PIGGY;
		packet.put(parms);
	}

	public String toString()
	{
		startPrint();
		addField("Length", Length);
		addField("Offset", Offset);
		addField("Quota", Quota);
		addField("Filename", Filename);
		addField("parms", parms);
		return getPrint();
	}
}

