/**
 * Copyright (C) 1998 Stewart Allen (stewart@neuron.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

public class SFTP
{
	// Values of Flags field in header
	public final static int ACKME    = 0x80000000;

	// Values of SEFlags field in header 
	public final static int MOREDATA = 0x1;  // on data packets, indicates more data to come 
	public final static int PIGGY    = 0x2;  // on RPC packets: piggybacked info present on this packet 
	public final static int ALLOVER  = 0x4;  // on RPC reply packets: indicates server got all data packets 
	public final static int TRIGGER  = 0x8;  // on ack packets: distinguishes a server "triggered" ack from a real one 
											 // necessary only for compatibility, triggers now send null timestamp echos 
	public final static int FIRST    = 0x10; // on data packets, indicates first of group sent by source 
	public final static int COUNTED  = 0x20; // on data packets: arrived or acked before last round 

	// SFTP Opcodes 
	public final static int START    = 1;    // Control: start sending data (flow is from RPC client to server)
	public final static int ACK      = 2;    // Control: acknowledgement you had requested 
	public final static int DATA     = 3;    // Data: next chunk; MOREDATA flag indicates whether EOF has been seen 
	public final static int NAK      = 4;    // Control: got a bogus packet from you 
	public final static int RESET    = 5;    // Control: reset transmission parameters 
	public final static int BUSY     = 6;    // Control: momentarily busy; reset your timeout counter 


	// Per-connection information: accessible via RPC2_GetSEPointer() and RPC2_SetSEPointer() 
	public final static int SFTPMAGIC     = 4902057;
	public final static int MAXOPACKETS   = 64;   // Maximum no of outstanding packets; multiple of 32 
	public final static int BITMASKWIDTH  = (MAXOPACKETS / 32);  // No of elements in integer array 
	public final static int MINDELTASS    = 10;   // minimum milliseconds between invocations of SendStrategy due to an ACK 


	// more stuff
	public final static int SFTPVERSION    = 3;
	public final static int MAXPACKETSIZE  = 2900; // (prefix+header+body) of largest sftp packet
	public final static int MAXBODYSIZE    = MAXPACKETSIZE - (60 * 4);

	public final static int DEFPACKETSIZE  = 2800;
	public final static int DEFWINDOWSIZE  = 32;
	public final static int DEFSENDAHEAD   = 8;

	public final static int MINPACKETSIZE  = 240;  // as above (1 IP fragment on SLIP)
	public final static int MINBODYSIZE    = MINPACKETSIZE - (60 * 4);
	public final static int MINWINDOWSIZE  = 2;
	public final static int MINSENDAHEAD   = 1;
}

