/**
 * Copyright (C) 1998 Stewart Allen (stewart@neuron.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

public class SFTP_Descriptor extends RPC2_BodyStruct
{
	// Transmission Direction (WhichWay)
	public final static int CLIENTTOSERVER=93;
	public final static int SERVERTOCLIENT=87;

	// FileInfoTag (Tag)
	public final static int FILEBYNAME=33;		// size = 264
	public final static int FILEBYINODE=58;		// size = 16
	public final static int FILEBYFD=67;		// size = 8
	public final static int FILEINVM=74;		// size = ??? + 8

	// size of C Value Union
	final static int ValueSize = 264;

	int TransmissionDirection;
	byte hashmark;
	long SeekOffset;
	long BytesTransferred;
	long ByteQuota;
	long QuotaExceeded;
	int Tag;

	// FileInfoTag (Tag)
	byte FileInfo[];


	// for CONSTRUCTING a packet
	public SFTP_Descriptor()
	{
	}

	// for READING a packet
	public SFTP_Descriptor(RPC2_Packet packet)
	{
		super(packet, "SFTP_Descriptor");
		TransmissionDirection = packet.getInt();
		hashmark = packet.getByte();
		SeekOffset = packet.getInt();
		BytesTransferred = packet.getInt();
		ByteQuota = packet.getInt();
		QuotaExceeded = packet.getInt();
		Tag = packet.getInt();
		FileInfo = packet.getBytes(ValueSize);
	}

	// for WRITING a packet
	public void encode(RPC2_Packet packet)
	{
	}

	public String toString()
	{
		startPrint();
		addField("TransmissionDirection", TransmissionDirection);
		addField("hashmark", hashmark);
		addField("SeekOffset", SeekOffset);
		addField("BytesTransferred", BytesTransferred);
		addField("ByteQuota", ByteQuota);
		addField("QuotaExceeded", ByteQuota);
		addField("Tag", Tag);
		addField("FileInfo", RPC2_Util.getHex(FileInfo));
		return getPrint();
	}
}

